import requests
from django.conf import settings

def _build_url(path):
    return "http://{host}:{port}{path}".format(
        host = getattr(settings, 'NGX_CACHE_PURGE_HOST', 'localhost'),
        port = getattr(settings, 'NGX_CACHE_PURGE_PORT', 80),
        path = path,
    )

class NginxManager(object):
    def purge(self, path):
        host_header = getattr(settings, 'NGX_CACHE_PURGE_HOST_HEADER', None)
        if host_header:
            response = requests.request("PURGE", _build_url(path), headers={'Host':host_header})
        else:
            response = requests.request("PURGE", _build_url(path))
        return response.status_code == 200



from django.db.models.signals import (
    post_save,
    post_delete,
    pre_delete,
    pre_save
)
from functools import partial
from cache_purge_hooks.manager import CacheManager
import logging


def __get_urls(instance, func):
    if func is not None:
        return func(instance)
    if hasattr(instance, 'get_absolute_urls'):
        return instance.get_absolute_urls()
    if hasattr(instance, 'get_absolute_url'):
        return [instance.get_absolute_url()]
    return []


def __pre_save_hook(model, func, instance, sender, **kwargs):
    """
    In case the url changes on the save, we need to save the
    old urls because those pages need to be invalidated.
    """
    try:
        old = sender.objects.get(pk=instance.pk)
    except sender.DoesNotExist:
        return
    instance.__cache_prehook_urls = set(__get_urls(old, func))


def __post_save_hook(model, func, instance, **kwargs):
    urls = set(__get_urls(instance, func))
    try:
        urls = urls.union(instance.__cache_prehook_urls)
    except AttributeError:
        pass
    with CacheManager() as cm:
        for url in urls:
            logging.info("expire:", url)
            cm.purge(url)


def __pre_delete_hook(model, func, instance, **kwargs):
    instance.__cache_prehook_urls = set(__get_urls(instance, func))


def __post_delete_hook(model, func, instance, **kwargs):
    urls = instance.__cache_prehook_urls
    with CacheManager() as cm:
        for url in urls:
            logging.info("expire:", url)
            cm.purge(url)


def cache_purge_hook(model, func=None):
    pre_save.connect(
        partial(__pre_save_hook, model, func),
        dispatch_uid="cache_purge_hook",
        sender=model,
        weak=False,
    )
    post_save.connect(
        partial(__post_save_hook, model, func),
        dispatch_uid="cache_purge_hook",
        sender=model,
        weak=False,
    )
    pre_delete.connect(
        partial(__pre_delete_hook, model, func),
        dispatch_uid="cache_purge_hook",
        sender=model,
        weak=False,
    )
    post_delete.connect(
        partial(__post_delete_hook, model, func),
        dispatch_uid="cache_purge_hook",
        sender=model,
        weak=False,
    )



import logging
import subprocess

from django.conf import settings

logger = logging.getLogger('django.cache_purge_hooks')

VARNISHADM_HOST = getattr(settings, 'VARNISHADM_HOST', 'localhost')
VARNISHADM_PORT = getattr(settings, 'VARNISHADM_PORT', 6082)
VARNISHADM_SECRET = getattr(settings, 'VARNISHADM_SECRET', '/etc/varnish/secret')
VARNISHADM_SITE_DOMAIN = getattr(settings, 'VARNISHADM_SITE_DOMAIN', '.*')
VARNISHADM_BIN = getattr(settings, 'VARNISHADM_ADM_BIN', '/usr/bin/varnishadm')

class VarnishManager(object):

    def purge(self, url):
        command = 'ban req.http.host ~ "{host}" && req.url ~ "{url}"'.format(
            host=VARNISHADM_SITE_DOMAIN.encode('ascii'),
            url=url.encode('ascii'),
        )
        self.send_command(command)

    def purge_all(self):
        self.purge('.*')

    def send_command(self, command):
        args = [VARNISHADM_BIN, '-S', VARNISHADM_SECRET, '-T', VARNISHADM_HOST+':'+str(VARNISHADM_PORT), command]
        try:
            subprocess.check_call(args)
        except subprocess.CalledProcessError as error:
            logger.error('Command "{0}" returned {1}'.format(' '.join(args), error.returncode))
            return False
        else:
            logger.debug('Command "{0}" executed successfully'.format(' '.join(args)))
            return True



from django.core.exceptions import ImproperlyConfigured
from importlib import import_module

# Note: This is a copy-paste from django 1.6 for backwards-compat reasons

def import_by_path(dotted_path, error_prefix=''):
    """
    Import a dotted module path and return the attribute/class designated by the
    last name in the path. Raise ImproperlyConfigured if something goes wrong.
    """
    try:
        module_path, class_name = dotted_path.rsplit('.', 1)
    except ValueError:
        raise ImproperlyConfigured("%s%s doesn't look like a module path" % (
            error_prefix, dotted_path))

    module = import_module(module_path)

    try:
        attr = getattr(module, class_name)
    except AttributeError:
        raise ImproperlyConfigured('%sModule "%s" does not define a "%s" attribute/class' % (
            error_prefix, module_path, class_name))
    return attr



from django.conf import settings
try:
    from django.utils import import_string
except ImportError:
    try:
        from django.utils import import_by_path as import_string
    except ImportError:
        from cache_purge_hooks.utils import import_by_path as import_string

def _get_manager():
    manager_cls =  import_string(settings.CACHE_PURGE_HOOKS_BACKEND)
    return manager_cls()

class CacheManager(object):
	def __init__(self):
		self.manager = _get_manager()

	def __enter__(self):
		return self

        def __exit__(self, *args):
            if hasattr(self.manager, 'close'):
                self.manager.close()

	def purge(self, command):
		self.manager.purge(command)

	def purge_all(self):
		self.manager.purge_all()



SECRET_KEY = "secret",
INSTALLED_APPS = [
    'sampleproject.sample',
]
DATABASES = {
    'default': {
      'ENGINE': 'django.db.backends.sqlite3',
      'NAME':':memory:'
    }
}
TEST_RUNNER = 'django_pytest.test_runner.run_tests'



from django.db import models
from cache_purge_hooks import cache_purge_hook

class TestModel(models.Model):
    a_field = models.CharField(max_length=20)

    def get_absolute_url(self):
        return '/foo'

cache_purge_hook(TestModel)



from setuptools import setup, find_packages

install_requires = ['requests']

try:
    import importlib
except ImportError:
    install_requires.append('importlib')


setup(
    name = 'django-cache-purge-hooks',
    version = '0.4.1',
    packages = find_packages(),
    author = 'Shu Zong Chen',
    author_email = 'shu.chen@freelancedreams.com',
    description = 'Drop-in cache (e.g. varnish) purging hooks for django',
    long_description = "Pluggable django app to purge caches.",
    license = "MIT",
    keywords = "django cache purge varnish nginx",
    install_requires = install_requires,
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    url = 'https://github.com/RealGeeks/django-cache-purge-hooks'
)
